function pathway=trimPathway(pathway, rxnsToKeep, deleteUnconnectedMets)
% trimPathway
%   Delete reactions from a pathway object
%
%   pathway                   map structure as generated by 
%                             constructPathwayFromCelldesigner
%   rxnsToKeep                cell array with the IDs of the reactions to
%                             keep
%   deleteUnconnectedMets     delete metabolites that are no longer connected
%                             by any reactions (opt, default true)
%
% 	pathway                   an updated pathway object
%
%   Usage: pathway=trimPathway(pathway, rxnsToKeep, deleteUnconnectedMets)
%
%   Rasmus Agren, 2010-12-16
%

if nargin<3
    deleteUnconnectedMets=true;
end

%First find the aliases and indexes of all reactions that should be deleted
indexesToDelete=false(numel(pathway.listOfSpecies),1);
aliasesToDelete={};

for i=1:numel(pathway.listOfSpecies)
   if strcmpi(pathway.listOfSpecies(i).type,'PROTEIN')
      %Check if the name is in the list of reactions to delete
      if ~ismember(pathway.listOfSpecies(i).name,rxnsToKeep)
          indexesToDelete(i)=true;
          aliasesToDelete=[aliasesToDelete;pathway.listOfSpecies(i).alias];
      end
   end
end

%Then go through each of the reactions and delete the ones that only have
%modifiers that are in the aliasesToDelete list
rxnsToDelete=false(numel(pathway.listOfReactions),1);

for i=1:numel(pathway.listOfReactions)
    %Keeps track of which components to delete. -1=enzyme to delete, 0=metabolite,
    %1=enzyme to keep
    componentsToDelete=zeros(numel(pathway.listOfReactions(i).componentList),1);
    for j=1:numel(pathway.listOfReactions(i).componentList)
        if strcmpi(pathway.listOfReactions(i).componentList(j).type,'ENZYME')
            %If the enzyme alias is not in the list then should it not be
            %deleted
            if ismember(pathway.listOfReactions(i).componentList(j).alias,aliasesToDelete)
                componentsToDelete(j)=-1;
            else
                componentsToDelete(j)=1;
            end
        end
    end
    %If all enzymes should be deleted then should the whole reacction be
    %deleted. Otherwise only delete those enzymes
    if ~any(componentsToDelete==1)
        rxnsToDelete(i)=true;
    else
        pathway.listOfReactions(i).componentList(componentsToDelete==-1)=[];
    end
end

%Delete the reactions and components that should be deleted
pathway.listOfSpecies(indexesToDelete)=[];
pathway.listOfReactions(rxnsToDelete)=[];

%This is not neat but it goes through all the reactions and adds the
%aliases that should be kept. All other are deleted
aliasesToKeeep={};
if deleteUnconnectedMets==true
    for i=1:numel(pathway.listOfReactions)
       for j=1:numel(pathway.listOfReactions(i).componentList)
           aliasesToKeeep=[aliasesToKeeep;pathway.listOfReactions(i).componentList(j).alias];
       end
    end
    
    %Go through the species again and only keep those that are in the
    %aliasesToKeeep list
    indexesToDelete=true(numel(pathway.listOfSpecies),1);
    for i=1:numel(pathway.listOfSpecies)
        if ismember(pathway.listOfSpecies(i).alias,aliasesToKeeep)
            indexesToDelete(i)=false;
        end
    end
    
    pathway.listOfSpecies(indexesToDelete)=[];
end


